@@ -4,11 +4,7 @@ import pandas as pd
 from pybaseball import statcast
 
 
-# TODO: note that pybaseball returns ALL games (not just regular season)
-# TODO: for book, should we filter to regular for any data directly provided?
-# TODO: download always gets ALL data, but loading can filer to only regular season
-# TODO: can pyarrow dtypes be used here? would that need to be changed in pybaseball?
-def download_seasons(start_year, end_year=None, force_download=False):
+def download_seasons(start_year, end_year=None, force_download=False, data_dir="statcast"):
 
     # define end_year if not provided by user, only download start year
     if end_year is None:
@@ -23,11 +19,11 @@ def download_seasons(start_year, end_year=None, force_download=False):
         raise ValueError("'start_year' must be less than or equal to 'end_year'.")
 
     # make statcast directory if it does not exist
-    os.makedirs("statcast", exist_ok=True)
+    os.makedirs(data_dir, exist_ok=True)
 
     # process all years requested
     for year in range(start_year, end_year + 1):
-        parquet_path = f"statcast/sc-{year}.parquet"
+        parquet_path = os.path.join(data_dir, f"sc-{year}.parquet")
         if os.path.exists(parquet_path) and not force_download:
             print(f"Data for {year} already exists, skipping.")
         else:
@@ -35,33 +31,22 @@ def download_seasons(start_year, end_year=None, force_download=False):
                 warnings.simplefilter("ignore", FutureWarning)
                 print(f"Downloading data for {year}.")
                 df = statcast(start_dt=f"{year}-01-01", end_dt=f"{year}-12-31")
-                df = df.convert_dtypes(dtype_backend="pyarrow")
-                df["game_date"] = pd.to_datetime(df["game_date"]).dt.date
-                df.to_parquet(parquet_path, index=False)
+                df = df.convert_dtypes(dtype_backend="numpy_nullable")
+                df.to_parquet(parquet_path, index=False, engine="pyarrow", compression="zstd")
 
 
-def load_season(year, force_download=False, regular_season=False):
-    parquet_path = f"statcast/sc-{year}.parquet"
-    if not os.path.exists(parquet_path):
-        print(f"Data for {year} does not exist, downloading.")
-        download_seasons(year, year, force_download=force_download)
-    if force_download:
-        print(f"Downloading data for {year}.")
-        download_seasons(year, year, force_download=force_download)
-    df = pd.read_parquet(parquet_path)
+def load_season(year, force_download=False, regular_season=False, data_dir="statcast"):
+    parquet_path = os.path.join(data_dir, f"sc-{year}.parquet")
+    if force_download or not os.path.exists(parquet_path):
+        download_seasons(year, year, force_download=force_download, data_dir=data_dir)
+    df = pd.read_parquet(parquet_path, engine="pyarrow")
     if regular_season:
         df = filter_regular_season(df)
     return df
 
 
-def filter_regular_season(df):
-    if "game_type" not in df.columns:
-        raise ValueError("DataFrame must contain a 'game_type' column.")
-    return df[df["game_type"] == "R"].copy()
-
-
 # all statcast features as of july 2025
-statcast_features = [
+statcast_features_default = [
     "pitch_type",
     "game_date",
     "release_speed",
@@ -249,11 +234,17 @@ statcast_features_reorder = [
     # swing tracking
     "bat_speed",
     "swing_length",
+    "attack_angle",
+    "attack_direction",
+    "swing_path_tilt",
+    "intercept_ball_minus_batter_pos_x_inches",
+    "intercept_ball_minus_batter_pos_y_inches",
     # batted ball tracking
     "launch_speed",
     "launch_angle",
     "estimated_woba_using_speedangle",
     "estimated_ba_using_speedangle",
+    "estimated_slg_using_speedangle",
     "hc_x",
     "hc_y",
     "hit_distance_sc",
@@ -273,14 +264,32 @@ statcast_features_reorder = [
 
 
 statcast_features_remove = [
-    feature for feature in statcast_features if feature not in statcast_features_reorder
+    feature for feature in statcast_features_default if feature not in statcast_features_reorder
 ]
 
-statcast_features_processed = statcast_features_reorder + statcast_features_remove
+statcast_features_ordered = statcast_features_reorder + statcast_features_remove
 
 
 def reorder_columns(df, keep_all=False):
-    df = df.loc[:, statcast_features_processed]
+    df = df.loc[:, statcast_features_ordered].copy()
     if not keep_all:
         df = df.drop(statcast_features_remove, axis=1, errors="ignore")
     return df
+
+
+# currently defaulting to ascending=False to match pybaseball
+def arrange_rows(df, keep_index=False, ascending=False):
+    sort_columns = ["game_date", "game_pk", "at_bat_number", "pitch_number"]
+    missing = [col for col in sort_columns if col not in df.columns]
+    if missing:
+        raise ValueError(f"DataFrame is missing required columns for sorting: {missing}")
+    df = df.sort_values(by=sort_columns, ascending=ascending)
+    if not keep_index:
+        df = df.reset_index(drop=True)
+    return df
+
+
+def filter_regular_season(df):
+    if "game_type" not in df.columns:
+        raise ValueError("DataFrame must contain a 'game_type' column.")
+    return df[df["game_type"] == "R"].copy()
