@@ -1,4 +1,6 @@
 import pytest
+import os
+import sys
 
 from abdwp.retrosheet import download_gamelogs
 from abdwp.retrosheet import load_gamelogs
@@ -48,3 +50,11 @@ def test_load_teams_runs():
         load_teams(2019)
     except Exception:
         pass
+
+
+@pytest.mark.skipif(sys.platform != "win32", reason="Only runs on Windows")
+def test_load_pbp_works():
+    file_dir = os.path.dirname(__file__)
+    download_events(2019, cwevent_dir=file_dir)
+    df = load_pbp(2019)
+    assert df is not None
